% MAIN_COMPARISON Main_comparison.m is a MatLab script which replicates the
% numerical results presented in A. Benfenati et al, "Proximal Approaches
% for Matrix Optimization Problems: Application to Robust Precision
% MatrixEstimation.", Section 4.4
clearvars
close all
clc

% Add the folders containing the m-files needed for the tests
addpath SideFuns/
addpath Funs/
addpath Solvers/
addpath Prox/

%% Parameters for test
load TEST_parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matrix A stored in TEST_parameters.mat contains the values for the
% regularization parameters for algorithms covsel, DR_matrix and MM set to
% obtain the results presented in Benfenati et al, "Proximal Approaches for
% Matrix Optimization Problems: Application to Robust Precision
% MatrixEstimation". The matrix A is 8x6:
%
% 1st column : noise levels from 0.1 to 0.8
% 2nd column : mu0 for g0 in MM approach
% 3rd column : mu1 for g1 in MM approach
% 4th column : mu0 for g0 in DR approach
% 5th column : mu1 for g1 in DR approach
% 6th column : mu1 for ell1 in GL approach
%
% The i-th row contains the regularization parameter for each noise level:
% for example, the 3rd row contains the regularization parameters for all 3
% algorithms for noise level equal to 0.3
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Set the seed and generated the seeds for different noise realizations for
% each level
rng('default');
Lmax = 10;
SEED = round(1e4*rand(8,Lmax));

% Allocating memory for storing the performance evaluators
% GLASSO
S_GL = zeros(Lmax,8);
F_GL = zeros(Lmax,8);
% MM
S_MM = zeros(Lmax,8);
F_MM = zeros(Lmax,8);
% DR
S_DR   = zeros(Lmax,8);
F_DR   = zeros(Lmax,8);

%% Main Cycle: for each noise level K Lmax noise realization are computed
for K = 1:8
    sigma  = A(K,1);
    for L = 1:Lmax
        fprintf('Noise: %1.2f - Realization: %2d\n',sigma,L)

        % Generating data for each realization of noise level. Notice that
        % since the eseed option is set to the default, the position of the
        % nonzero entries is always the same. The nseed options is set for
        % having Lmax different realization of the noise level sigma
        [SIGMA, ISIGMA, S]  = GenerateData('NOISE',sigma,'nseed',SEED(K,L));
        zero_obj            = sum(ISIGMA(:)==0);
        
        %% BOYD
        mu1_boyd     = A(K,6);
        [X, history] = covsel(S, mu1_boyd, 1, 1,'QUIET',1);
        S_GL(L,K)    = norm(inv(X)-SIGMA,'fro')/norm(SIGMA,'fro');
        F_GL(L,K)    = sum(X(:)~=0 & ISIGMA(:)==0)/zero_obj;
        
        %% MMDR
        J = rand(size(S));
        J = J*J' + eye(size(J));
        
        optionsMMDR.mu0         = A(K,2);
        optionsMMDR.mu0b        = 1;
        optionsMMDR.mu1         = A(K,3);
        optionsMMDR.gamma       = 1;         % Prox parameter
        optionsMMDR.INNITER     = 2000;      % number of iterations
        optionsMMDR.innstop     = 2;         % stopping criterion
        optionsMMDR.inntol      = 1e-10;     % tolerance for the criterion
        optionsMMDR.EXTITER     = 20;        % number of iterations
        optionsMMDR.extstop     = 2;         % stopping criterion
        optionsMMDR.exttol      = 1e-8;      % tolerance for the criterion
        optionsMMDR.lambda      = 1;
        optionsMMDR.F           = 'LogDet';
        optionsMMDR.g0          = 'iSchatten1';
        optionsMMDR.init        = J;         %S + eye(size(S));
        optionsMMDR.params      = [A(K,2), sigma];
        
        [C_fin, Cn, fobjG, Q, t_vecG, RMSEC] = MMDR(optionsMMDR.init,...
                optionsMMDR.gamma, A(K,2), A(K,3), S, sigma, ...
                'INNITER', optionsMMDR.INNITER,...
                'INNSTOP', optionsMMDR.innstop,...
                'INNTOL',  optionsMMDR.inntol,...
                'EXTITER', optionsMMDR.EXTITER,...
                'EXTSTOP', optionsMMDR.extstop,...
                'EXTTOL',  optionsMMDR.exttol,...
                'LAMBDA',  optionsMMDR.lambda,...
                'F',       optionsMMDR.F,...
                'G0',      optionsMMDR.g0,...
                'OBJ',     ISIGMA,...
                'verbose', 0,...
                'PARAMS',  optionsMMDR.params);
            
        S_MM(L,K) = norm(inv(C_fin)-SIGMA,'fro')/norm(SIGMA,'fro');
        F_MM(L,K) = sum(C_fin(:)~=0 & ISIGMA(:)==0)/zero_obj;
        
        %% DR
        optionsDR               = optionsMMDR;
        optionsDR.mu0           = A(K,4);
        optionsDR.mu1           = A(K,5);
        optionsDR.params        = A(K,4);
        
        [Cf_DR, ~,fobjDR, t_vecDR, FPR, TPR, RMSE] = DR_matrix(optionsDR.init,...
            optionsDR.gamma, optionsDR.mu0, optionsDR.mu1, S,...
            'ITER',    optionsDR.INNITER,...
            'STOP',    optionsDR.innstop,...
            'TOL',     optionsDR.inntol,...
            'LAMBDA',  optionsDR.lambda,...
            'F',       optionsDR.F,...
            'G0',      optionsDR.g0,...
            'OBJ',     ISIGMA,...
            'VERBOSE', 0,...
            'PARAMS',  optionsDR.params);
        
        S_DR(L,K) = norm(inv(Cf_DR)-SIGMA,'fro')/norm(SIGMA,'fro');
        F_DR(L,K) = FPR(end);
        
    end
end

%% Plotting the result
h = plot(1:8,mean(S_MM,1),'-o','Linewidth',2);
h.MarkerFaceColor = h.Color;
hold on
h = plot(1:8,mean(S_DR,1),':>','Linewidth',2);
h.MarkerFaceColor = h.Color;
h = plot(1:8,mean(S_GL,1),'--<','Linewidth',2);
h.MarkerFaceColor = h.Color;
legend('MM','DR','GLASSO')
title('Covariance rmse')
axis([1 8 0 1])

figure
h = plot(1:8,mean(F_MM,1),'-o','Linewidth',2);
h.MarkerFaceColor = h.Color;
hold on
h = plot(1:8,mean(F_DR,1),':>','Linewidth',2);
h.MarkerFaceColor = h.Color;
h = plot(1:8,mean(F_GL,1),'--<','Linewidth',2);
h.MarkerFaceColor = h.Color;
legend('MM','DR','GLASSO')
title('fpr')
axis([1 8 0 .45])


%==========================================================================
%
% Version : 1.0 (04-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%
