 function p = prox_Froben_Schatten2(x, gamma, varargin)
% PROX_FROBEN_SCHATTEN2 Compute the proximity operator of the Frobenius
% norm coupled with the Schatten-2 norm.
%
% [y] = prox_Froben_Schatten2(x,gamma,varargin)
%
% This function finds the proximal point of
%
%  phi(z) = 1/2 * gamma * |z|^2 + gamma * mu * |z|^2 
%
% The parameter mu is optional, its default value is set to 1.
%
%--------------------------------------------------------------------------
% MANDATORY INPUT
%
% x        : (double array) argument of the prox operator
% gamma    : (double)       parameter of the proximal operator
%
%--------------------------------------------------------------------------
% OPTIONAL INPUT
%
% varargin : (double) further parameter
%
%--------------------------------------------------------------------------
% OUTPUT
%
% y        : (double array) proximal point
%
%==========================================================================
%
% Version : 1.0 (04-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%


% check input
if any( gamma(:) <= 0 ) || ~isscalar(gamma) && any(size(gamma) ~= size(x))
    error('''gamma'' must be positive and either scalar or the same size as ''x''')
end
%-----%

mu = 1;

if ~isempty(varargin)
    mu = varargin{1}(1);
end

% compute the prox
p = max(x/(1+gamma+2*mu),eps);