function y = prox_LogDet_Schatten2(x, gamma, varargin)
% PROX_LOGDET_SCHATTEN2 Compute the proximity operator of the logdet
% coupled with the Schatten-1 norm.
%
% [y] = prox_Froben_Schatten1(x,gamma,varargin)
%
% This function finds the proximal point of
%
% phi(z) =  -gamma * sum( log (z) ) + gamma * mu * |z|^2
%
% The input variable varargin can be used both for classical approach and
% for the MM approach described in Benfenati et al, "Proximal Approaches
% for Matrix Optimization Problems: Application to Robust Precision
% MatrixEstimation".
%
%--------------------------------------------------------------------------
% MANDATORY INPUT
%
% x        : (double array) argument of the prox operator
% gamma    : (double)       parameter of the proximal operator
%
%--------------------------------------------------------------------------
% OPTIONAL INPUT
%
% varargin : (double) further parameter
%
%--------------------------------------------------------------------------
% OUTPUT
%
% y        : (double array) proximal point
%
%==========================================================================
%
% Version : 1.0 (04-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%

% Default values
mu = 1;
s  = 0;

if ~isempty(varargin)
    switch length(varargin{:})
        case 1
            mu = varargin{1}(1);
            s  = 0;
        case 2
            mu = varargin{1}(1);
            s  = varargin{1}(2)^2;
    end
end



% check input
if any( gamma(:) <= 0 ) || ~isscalar(gamma) && any(size(gamma) ~= size(x))
    error('''gamma'' must be positive and either scalar or the same size as ''x''')
end
%-----%

% compute the prox
y = zeros(size(x));
for l = 1:length(x)
    coeff = [s*(2*gamma*mu+1), 1-x(l)*s+2*gamma*mu, -x(l), -gamma];
    a = roots(coeff);
    a = a(a>0);
    y(l) = a;
end
