 function [y,g] = prox_svd(x, gamma, prox_phi, varargin)
% PROX_SVD Compute the proximity operator of gamma*phi
%
% [y,g] = prox_svd(x, gamma, prox_phi, varargin)
%
% This procedure computes the proximity operator of the matrix function:
%
%   f(X) = gamma * phi(s)     with   X = U' * diag(s) * U
% 
% It is assumed that the matrix is squared. This function is a slight
% modification of the one available at http://proximity-operator.net
%
% -------------------------------------------------------------------------
% MANDATORY INPUT
% 
% x        : (double array) argument of the prox operator
% gamma    : (double)       parameter of the proximal operator 
% prox_phi : (function handle) handle to the function 
%
%--------------------------------------------------------------------------
%
% OPTIONAL INPUT
%
% varargin : (cell) additional parameters for the function 'prox_phi'
%
%--------------------------------------------------------------------------
% OUTPUT
%
% y        : (double array) matricial proximal point
% g        : (double array) eigenvalues  of the proximal point%
%
%==========================================================================
%
% Version : 1.0 (04-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%

% spectral decomposition
[U, s] = eig(x);
s      = diag(s);

% prox computation
g = prox_phi(s, gamma, varargin{:});

% spectral reconstruction
y = U*diag(g)*U';
