function [Ck12, Ck,fobj, t_vec, FPR, TPR, ERR] = DR_matrix(Ck, gamma, mu0, mu1, S, varargin)
% DR_MATRIX Matrix estimation via the Douglas-Rachford algorithm
%
% [Ck12, Ck,fobj, t_vec, FPR, TPR, ERR] = DR_matrix(Ck, gamma, mu0, mu1, S,
% varargin)
% 
% Solves the following problem
%
% argmin_C D[f](C)  +mu_0*g_0(C) + mu_1*g_1(C)
%
% D[f] stands for the Bregman distance of the function f, g0 has to be a
% spectral function, while g1 could act on the whole matrix C. See A.
% Benfenati et al, "Proximal Approaches for Matrix Optimization Problems:
% Application to Robust Precision MatrixEstimation." for technical details.
%
% The functions f, g0 and g1 can be chosen among several options: the
% default are -logdet, Schatten--1 norm and component-wise l_1 norm,
% respectively. The computations of the functionals and of the proximity
% operators are managed by functions handles. 
%
% -------------------------------------------------------------------------
% MANDATORY INPUT
% Ck      : (double array) initial estimate
% gamma   : (double)       parameter for the proximal operator
% mu0     : (double)       regularization parameter for g0
% mu1     : (double)       regularization parameter for g1
% S       : (double array) given symmetric semidefinite positive matrix
%
% -------------------------------------------------------------------------
% OPTIONAL INPUT
%
% ITER    : (integer)      maximum number of iterations.
%                          Default: 1000;
% STOP    : (integer)      stopping criterion for DR. Option 1 stands for
%                          running all the iterations; options 2 stands
%                          for controlling the relative decreasing of the
%                          objective function value below the tolerance
%                          inn_tol. Options 3 stands for checking the
%                          relative distance between two successive
%                          iterations. Default: 1;
% LAMBDA  : (double)       DR parameter.
%                          Default: 1.5.
% F       : (string)       Fit-to-data functional.
%                          Default: "LogDet".
% G0      : (string)       eigenvalues regularization functional,
%                          Default: Shatten--1 norm.
% G1      : (string)       regularization functional,
%                          Default: l_1 norm.
% OBJ     : (double array) Benchmark for computing performances.
%                          Default: empty.
% VERBOSE : (integer)      Verbosity. Option 1 (or greater than 0) prints
%                          iterations 'information on the shell. Option 0
%                          stands for silence. Default: 1.
% PARAMS  : (double array) parameters needed for computing g0 and its
%                          proximal operators. See the help of the related
%                          function
% 
% -------------------------------------------------------------------------
% OUTPUT
%
% Ck      : (double array) estimated solution
% Ck12    : (double array) estimated solution (At convergence, Ck=Ck12)
% fobj    : (double array) vector of functional's values
% t_vec   : (double array) employed time per iteration
% FPR     : (double array) if obj is not empty, false positive rate wrt obj
% TPR     : (double array) if obj is not empty, true positive rate wrt obj
% ERR     : (double array) if obj is not empty, relative reconstruction 
%                          error wrt obj
%
%
%==========================================================================
%
% Version : 1.0 (04-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%

% Default values
NITER   = 1000;
stop    = 1;
tol     = 1e-6;
lambda  = 1.5;
FitData = 'LogDet';
g0      = 'Schatten1';
g1      = 'ell1';
verbose = 1;
params  = [mu0 0];
obj     = [];

if (nargin-length(varargin)) ~= 5
    error('Wrong number of required parameters');
end

if (rem(length(varargin),2)==1)
    error('Optional parameters should always go by pairs');
    fprintf('!\n');
else
    for i=1:2:(length(varargin)-1)
        switch upper(varargin{i})
            case 'ITER'
                NITER = varargin{i+1};
            case 'STOP'
                stop = varargin{i+1};
            case 'TOL'
                tol = varargin{i+1};
            case 'LAMBDA'
                lambda = varargin{i+1};
            case 'F'
                FitData = varargin{i+1};
            case 'G0'
                g0 = varargin{i+1};
            case 'G1'
                g1 = varargin{i+1};
            case 'VERBOSE'
                verbose = varargin{i+1};
            case 'OBJ'
                obj = varargin{i+1};
            case 'PARAMS'
                params = varargin{i+1};
                
        end
    end
end

% Function Handles
prox_g0  = str2func(sprintf('@prox_%s_%s',FitData,g0));
prox_g1  = str2func(sprintf('@prox_%s',g1));
Fit2Data = str2func(FitData);
g0       = str2func(g0);
g1       = str2func(g1);

% First estimation
% Computations to be done once
if strcmp(FitData,'LogDet')
    gS = -gamma*S;
else
    gS = gamma*S;
end
% [~,dc]  = mexeig(Ck);
% dc      = diag(dc);
dc = eig(Ck);

fobj        = zeros(NITER+1,1);
t_vec       = zeros(NITER+1,1);
TPR         = zeros(NITER+1,1);
FPR         = zeros(NITER+1,1);
ERR         = zeros(NITER+1,1);

fobj(1)     = Fit2Data(Ck,S,dc) + mu0*g0(dc,params) + mu1*g1(Ck);

if ~isempty(obj)
    nzero_obj   = sum(obj(:)~=0);
    zero_obj    = sum(obj(:)==0);
    norm_obj    = norm(obj,'fro');
    ERR(1)      = norm(Ck-obj,'fro')/norm_obj;
    TPR(1)      = sum(Ck(:)~=0 & obj(:)~=0)/nzero_obj;
    FPR(1)      = sum(Ck(:)~=0 & obj(:)==0)/zero_obj;
end

t_vec(1)    = 0;

if verbose
    fprintf('%5d) Fobj: %+4.8f ',0,fobj(1));
    fprintf('TPR : %2.2f FPR : %2.2f RMSE : %2.2f\n',TPR(1),FPR(1),ERR(1));
end

i       = 1;
loop    = 1;

tic;
while loop
    
    %% First Operator
    Ck12 = prox_g1(Ck + gS,gamma,mu1);

    %% Second operator
    temp = 2*Ck12-Ck;
    [temp] = prox_svd(temp, gamma, prox_g0, params);
    
    %% Final update
    Ck = Ck + lambda * (temp - Ck12);
    t_vec(i+1) = toc;
    
    %% Computing the Objective
    fobj(i+1) =  Fit2Data(Ck12,S) + mu0*g0(Ck12,params) + mu1*g1(Ck12);
    if ~isempty(obj)
        ERR(i+1)  = norm(Ck12-obj,'fro')/norm_obj;
        TPR(i+1)  = sum(Ck12(:)~=0 & obj(:)~=0)/nzero_obj;
        FPR(i+1)  = sum(Ck12(:)~=0 & obj(:)==0)/zero_obj;
    end
    
    if verbose
        fprintf('%5d) Fobj: %+4.8f ',i,fobj(i+1));
        if ~isempty(obj)
            fprintf('TPR : %2.2f FPR : %2.2f RMSE : %2.2f',TPR(i+1)*100,FPR(i+1)*100,ERR(i+1)*100);
        end
        fprintf('\n');
    end
    
    switch stop
        case 1
            % Number of iterations
            if i>NITER-1
                loop = 0;
            end
        case 2
            % Relative decrease of the objective
            if ((abs(fobj(i+1)-fobj(i))/abs(fobj(i))<tol)) || (i>NITER-1)
                loop = 0;
            end
        otherwise  % Maybe this check has to be done at the beginning
            error('Invalid criterion.');
    end
    i = i+1;
    
end

% Cutting off the extra elements
fobj(i+1:end)   = [];
t_vec(i+1:end)  = [];
ERR(i+1:end)    = [];
TPR(i+1:end)    = [];
FPR(i+1:end)    = [];
end

