function y = LogDet(C,S,varargin)
% LOGDET Compute the Bregman distance of the negative logarithmic
% determinant between two matrices.
%
%  y = LogDet(C,S,varargin)
%
% It computes the Bregman distance of the logdet function of C wrt S. When
% the 3rd argumnet is passed, the determinant computation is avoided and
% the spectral property of the logdet is exploited. The matrices are stull
% needed for the trace operator.
%
% -------------------------------------------------------------------------
% MANDATORY INPUT
%
% C        : (double array) 1st argument 
% S        : (double array) 2nd argument 
%
% -------------------------------------------------------------------------
% OPTIONAL INPUT
%
% varargin : (double array) eigenvalues of C. There is no control if the
%                           provided vector actually corresponds to the
%                           eigenvalues of C
% -------------------------------------------------------------------------
% 
% OUTPUT
%
% y        : (double)       value of the Bregman distance
%
%
%==========================================================================
%
% Version : 1.0 (04-06-2019)
% Author  : A. Benfenati (alessandro.benfenati@unimi.it)
%
%==========================================================================
%
% COPYRIGHT NOTIFICATION
%
% Permission to copy and modify this software and its documentation for
% internal research use is granted, provided that this notice is retained
% thereon and on all copies or modifications. The authors and their
% respective Universities makes no representations as to the suitability
% and operability of this software for any purpose. It is provided "as is"
% without express or implied warranty. Use of this software for commercial
% purposes is expressly prohibited without contacting the authors.
%
% This program is free software; you can redistribute it and/or modify it
% under the terms of the GNU General Public License as published by the
% Free Software Foundation; either version 3 of the License, or (at your
% option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
% Public License for more details.
%
% You should have received a copy of the GNU General Public License along
% with this program; if not, either visite http://www.gnu.org/licenses/ or
% write to Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
% 02139, USA.
%

dimC = size(C);
dimS = size(S);

% Dimension Check
if ismember(1,dimC ~= dimS | size(dimS)>2 | size(dimC)>2) 
    error('Dimension not consistent');
end


if ~isempty(varargin)
    % Dimension Check
    if length(varargin{1})~=dimC(1)
            error('Dimension not consistent');
    end
    y = -sum(log(varargin{1})) + trace(C*S);
else 
    Dc = det(C);
    if Dc - 0 < 1e-323
        fprintf('Warning: determinant close to zero\n');
    end
    y = -log(Dc) + trace(C*S);
end

